@@ -14,19 +14,37 @@
 cSoppalusikkaLogoCache SoppalusikkaLogoCache(0);
 
 cSoppalusikkaLogoCache::cSoppalusikkaLogoCache(unsigned int cacheSizeP)
-: cacheSizeM(cacheSizeP), bitmapM(NULL)
+: xFactorM(1.0),
+  yFactorM(1.0),
+  antiAliasM(false),
+  cacheSizeM(cacheSizeP),
+  bitmapM(NULL)
 {
 }
 
 cSoppalusikkaLogoCache::~cSoppalusikkaLogoCache()
 {
-  // let's flush the cache
   Flush();
 }
 
+void cSoppalusikkaLogoCache::Refresh()
+{
+  int width, height;
+  double aspect, xfactor, yfactor;
+  cDevice::PrimaryDevice()->GetOsdSize(width, height, aspect);
+  debug("%s(): %dx%d", __PRETTY_FUNCTION__, width, height);
+  xfactor = (double)width / DEFAULT_OSD_WIDTH;
+  yfactor = (double)height / DEFAULT_OSD_HEIGHT;
+  if (!DoubleEqual(xfactor, xFactorM) || !DoubleEqual(yfactor, yFactorM)) {
+     xFactorM = xfactor;
+     yFactorM = yfactor;
+     Flush();
+     }
+}
+
 bool cSoppalusikkaLogoCache::Resize(unsigned int cacheSizeP)
 {
-  debug("cPluginSkinSoppalusikka::Resize(%d)", cacheSizeP);
+  debug("%s(%d)", __PRETTY_FUNCTION__, cacheSizeP);
   // flush cache only if it's smaller than before
   if (cacheSizeP < cacheSizeM) {
      Flush();
@@ -43,15 +61,15 @@ bool cSoppalusikkaLogoCache::Load(const char *fileNameP)
      return false;
   // replace '/' characters with '~'
   strreplace(fileName, '/', '~');
-  debug("cPluginSkinSoppalusikka::Load(%s)", fileName);
+  debug("%s(%s)", __PRETTY_FUNCTION__, fileName);
   // does the logo exist already in map
   std::map<std::string, cBitmap*>::iterator i = cacheMapM.find(fileName);
   if (i != cacheMapM.end()) {
      // yes - cache hit!
-     debug("cPluginSkinSoppalusikka::Load() CACHE HIT!");
+     debug("%s() CACHE HIT!", __PRETTY_FUNCTION__);
      // check if logo really exist
      if (i->second == NULL) {
-        debug("cPluginSkinSoppalusikka::Load() EMPTY");
+        debug("%s() EMPTY", __PRETTY_FUNCTION__);
         // empty logo in cache
         free(fileName);
         return false;
@@ -60,7 +78,7 @@ bool cSoppalusikkaLogoCache::Load(const char *fileNameP)
      }
   else {
      // no - cache miss!
-     debug("cPluginSkinSoppalusikka::Load() CACHE MISS!");
+     debug("%s() CACHE MISS!", __PRETTY_FUNCTION__);
      // try to load xpm logo
      LoadXpm(fileName);
      // check if cache is active
@@ -68,7 +86,7 @@ bool cSoppalusikkaLogoCache::Load(const char *fileNameP)
         // update map
         if (cacheMapM.size() >= cacheSizeM) {
            // cache full - remove first
-           debug("cPluginSkinSoppalusikka::Load() DELETE");
+           debug("%s() DELETE", __PRETTY_FUNCTION__);
            if (cacheMapM.begin()->second != NULL) {
               // logo exists - delete it
               cBitmap *bmp = cacheMapM.begin()->second;
@@ -78,12 +96,12 @@ bool cSoppalusikkaLogoCache::Load(const char *fileNameP)
            cacheMapM.erase(cacheMapM.begin());
            }
         // insert logo into map
-        debug("cPluginSkinSoppalusikka::Load() INSERT(%s)", fileName);
+        debug("%s() INSERT(%s)", fileName, __PRETTY_FUNCTION__);
         cacheMapM.insert(std::make_pair(fileName, bitmapM));
         }
      // check if logo really exist
      if (bitmapM == NULL) {
-        debug("cPluginSkinSoppalusikka::Load() EMPTY");
+        debug("%s() EMPTY", __PRETTY_FUNCTION__);
         // empty logo in cache
         free(fileName);
         return false;
@@ -105,37 +123,39 @@ bool cSoppalusikkaLogoCache::LoadXpm(const char *fileNameP)
 
   // create absolute filename
   cString filename = cString::sprintf("%s/%s.xpm", SoppalusikkaConfig.GetLogoDir(), fileNameP);
-  debug("cPluginSkinSoppalusikka::LoadXpm(%s)", *filename);
+  debug("%s(%s)", __PRETTY_FUNCTION__, *filename);
   // check validity
   if ((stat(*filename, &stbuf) == 0) && bmp->LoadXpm(*filename)) {
-     if ((bmp->Width() == ChannelLogoWidth) && (bmp->Height() == ChannelLogoHeight)) {
-        debug("cPluginSkinSoppalusikka::LoadXpm() LOGO FOUND");
-        // assign bitmap
-        bitmapM = bmp;
-        return true;
+     // assign bitmap
+     debug("%s() LOGO FOUND", __PRETTY_FUNCTION__);
+     if (!DoubleEqual(1.0, xFactorM) || !DoubleEqual(1.0, yFactorM) || (bmp->Width() != DEFAULT_LOGO_WIDTH) || (bmp->Height() != DEFAULT_LOGO_HEIGHT)) {
+        cBitmap *tmp = bmp;
+        // scale bitmap keeping default proportions
+        bmp = bmp->Scaled(yFactorM * DEFAULT_LOGO_WIDTH / bmp->Width(), yFactorM * DEFAULT_LOGO_HEIGHT / bmp->Height(), bmp->Bpp() > 4);
+        debug("%s() LOGO SCALED", __PRETTY_FUNCTION__);
+        DELETENULL(tmp);
         }
-     else
-        error("Invalid logo resolution in '%s'", *filename);
+     bitmapM = bmp;
+     return true;
      }
   // no valid xpm logo found - delete bitmap
-  debug("cPluginSkinSoppalusikka::LoadXpm() LOGO NOT FOUND OR INVALID RESOLUTION");
-  delete bmp;
+  debug("%s() LOGO NOT FOUND OR INVALID RESOLUTION", __PRETTY_FUNCTION__);
+  DELETENULL(bmp);
   bitmapM = NULL;
   return false;
 }
 
 bool cSoppalusikkaLogoCache::Flush(void)
 {
-  debug("cPluginSkinSoppalusikka::Flush()");
+  debug("%s()", __PRETTY_FUNCTION__);
   // check if map is empty
   if (!cacheMapM.empty()) {
-     debug("cPluginSkinSoppalusikka::Flush() NON-EMPTY");
+     debug("%s() NON-EMPTY", __PRETTY_FUNCTION__);
      // delete bitmaps and clear map
      for (std::map<std::string, cBitmap*>::iterator i = cacheMapM.begin(); i != cacheMapM.end(); ++i) {
          cBitmap *bmp = i->second;
-         if (bmp)
-            DELETENULL(bmp);
          cacheMapM.erase(i);
+         DELETENULL(bmp);
          }
      // nullify bitmap pointer
      bitmapM = NULL;
