@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: channels.c 2.16 2010/06/05 13:33:57 kls Exp $
+ * $Id: channels.c 2.19 2011/08/26 12:44:21 kls Exp $
  */
 
 #include "channels.h"
@@ -262,12 +262,14 @@ static int IntArraysDiffer(const int *a, const int *b, const char na[][MAXLANGCO
 {
   int result = 0;
   for (int i = 0; a[i] || b[i]; i++) {
-      if (a[i] && na && nb && strcmp(na[i], nb[i]) != 0)
+      if (!a[i] || !b[i]) {
+         result |= VALDIFF;
+         break;
+         }
+      if (na && nb && strcmp(na[i], nb[i]) != 0)
          result |= STRDIFF;
       if (a[i] != b[i])
          result |= VALDIFF;
-      if (!a[i] || !b[i])
-         break;
       }
   return result;
 }
@@ -379,9 +381,9 @@ void cChannel::SetSubtitlingDescriptors(uchar *SubtitlingTypes, uint16_t *Compos
 void cChannel::SetTeletextSubtitlePages(tTeletextSubtitlePage pages[], int numberOfPages)
 {
   int mod = CHANNELMOD_NONE;
-  if (totalTtxtSubtitlePages != (fixedTtxtSubtitlePages + numberOfPages))
+  if (totalTtxtSubtitlePages != numberOfPages)
      mod |= CHANNELMOD_PIDS;
-  totalTtxtSubtitlePages = fixedTtxtSubtitlePages;
+  totalTtxtSubtitlePages = 0;
   for (int i = 0; (i < numberOfPages) && (totalTtxtSubtitlePages < MAXTXTPAGES); i++) {
       if (teletextSubtitlePages[totalTtxtSubtitlePages].ttxtMagazine != pages[i].ttxtMagazine ||
           teletextSubtitlePages[totalTtxtSubtitlePages].ttxtPage != pages[i].ttxtPage ||
@@ -515,8 +517,8 @@ cString cChannel::ToText(const cChannel *Channel)
      if (Channel->vpid && Channel->vtype)
         q += snprintf(q, sizeof(vpidbuf) - (q - vpidbuf), "=%d", Channel->vtype);
      *q = 0;
-     const int BufferSize = (MAXAPIDS + MAXDPIDS) * (5 + 1 + MAXLANGCODE2 + 5) + 10; // 5 digits plus delimiting ',' or ';' plus optional '=cod+cod@type', +10: paranoia
-     char apidbuf[BufferSize];
+     const int ABufferSize = (MAXAPIDS + MAXDPIDS) * (5 + 1 + MAXLANGCODE2 + 5) + 10; // 5 digits plus delimiting ',' or ';' plus optional '=cod+cod@type', +10: paranoia
+     char apidbuf[ABufferSize];
      q = apidbuf;
      q += IntArrayToString(q, Channel->apids, 10, Channel->alangs, Channel->atypes);
      if (Channel->dpids[0]) {
@@ -524,16 +526,13 @@ cString cChannel::ToText(const cChannel *Channel)
         q += IntArrayToString(q, Channel->dpids, 10, Channel->dlangs, Channel->dtypes);
         }
      *q = 0;
-     const int TBufferSize = 5 + 1 + (MAXTXTPAGES * (3 + 1 + MAXLANGCODE1 + 1)) + 10; // '12345;150=deu,151=fin,...', +10: paranoia
+     const int TBufferSize = MAXSPIDS * (5 + 1 + MAXLANGCODE2) + 10; // 5 digits plus delimiting ',' or ';' plus optional '=cod+cod', +10: paranoia and tpid
      char tpidbuf[TBufferSize];
      q = tpidbuf;
      q += snprintf(q, sizeof(tpidbuf), "%d", Channel->tpid);
-     if (Channel->fixedTtxtSubtitlePages > 0) {
-        q += snprintf(q, sizeof(tpidbuf) - (q - tpidbuf), ";");
-        for (int i = 0; i < Channel->fixedTtxtSubtitlePages; ++i) {
-            tTeletextSubtitlePage page = Channel->teletextSubtitlePages[i];
-            q += snprintf(q, sizeof(tpidbuf) - (q - tpidbuf), "%d=%s", page.PageNumber(), page.ttxtLanguage);
-            }
+     if (Channel->spids[0]) {
+        *q++ = ';';
+        q += IntArrayToString(q, Channel->spids, 10, Channel->slangs);
         }
      char caidbuf[MAXCAIDS * 5 + 10]; // 5: 4 digits plus delimiting ',', 10: paranoia
      q = caidbuf;
@@ -582,6 +581,8 @@ bool cChannel::Parse(const char *s)
            sid = atoi(caidbuf);
            delete caidbuf;
            caidbuf = NULL;
+           if (sscanf(tpidbuf, "%d", &tpid) != 1)
+              return false;
            caids[0] = tpid;
            caids[1] = 0;
            tpid = 0;
@@ -592,6 +593,7 @@ bool cChannel::Parse(const char *s)
         atypes[0] = 0;
         dpids[0] = 0;
         dtypes[0] = 0;
+        spids[0] = 0;
         ok = false;
         if (parambuf && sourcebuf && vpidbuf && apidbuf) {
            parameters = parambuf;
@@ -674,37 +676,30 @@ bool cChannel::Parse(const char *s)
               dpids[NumDpids] = 0;
               dtypes[NumDpids] = 0;
               }
-           if (tpidbuf) {
-              char *p;
-              fixedTtxtSubtitlePages = 0;
-              // 2001;150=deu,151=fin
-              if ((p = strchr(tpidbuf, ';')) != NULL) {
-                 char *q, *strtok_next;
-                 *p++ = 0;
-                 while ((q = strtok_r(p, ",", &strtok_next)) != NULL) {
-                       if (fixedTtxtSubtitlePages < MAXTXTPAGES) {
-                          int page;
-                          char *l = strchr(q, '=');
-                          if (l)
-                             *l++ = 0;
-                          if (sscanf(q, "%d", &page) == 1) {
-                             teletextSubtitlePages[fixedTtxtSubtitlePages] = tTeletextSubtitlePage(page);
-                             if (l)
-                                strn0cpy(teletextSubtitlePages[fixedTtxtSubtitlePages].ttxtLanguage, l, MAXLANGCODE1);
-                             fixedTtxtSubtitlePages++;
-                             }
-                          else
-                             esyslog("ERROR: invalid Teletext page!"); // no need to set ok to 'false'
+           int NumSpids = 0;
+           if ((p = strchr(tpidbuf, ';')) != NULL) {
+              *p++ = 0;
+              char *q;
+              char *strtok_next;
+              while ((q = strtok_r(p, ",", &strtok_next)) != NULL) {
+                    if (NumSpids < MAXSPIDS) {
+                       char *l = strchr(q, '=');
+                       if (l) {
+                          *l++ = 0;
+                          strn0cpy(slangs[NumSpids], l, MAXLANGCODE2);
                           }
                        else
-                          esyslog("ERROR: too many Teletext pages!"); // no need to set ok to 'false'
-                       p = NULL;
+                          *slangs[NumSpids] = 0;
+                       spids[NumSpids++] = strtol(q, NULL, 10);
                        }
-                 totalTtxtSubtitlePages = fixedTtxtSubtitlePages;
-                 }
-              if (sscanf(tpidbuf, "%d", &tpid) != 1)
-                 return false;
+                    else
+                       esyslog("ERROR: too many SPIDs!"); // no need to set ok to 'false'
+                    p = NULL;
+                    }
+              spids[NumSpids] = 0;
               }
+           if (sscanf(tpidbuf, "%d", &tpid) != 1)
+              return false;
            if (caidbuf) {
               char *p = caidbuf;
               char *q;
