@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h 2.28 2010/09/12 11:31:21 kls Exp $
+ * $Id: config.h 2.34 2011/08/20 08:51:47 kls Exp $
  */
 
 #ifndef __CONFIG_H
@@ -22,13 +22,13 @@
 
 // VDR's own version number:
 
-#define VDRVERSION  "1.7.16"
-#define VDRVERSNUM   10716  // Version * 10000 + Major * 100 + Minor
+#define VDRVERSION  "1.7.21"
+#define VDRVERSNUM   10721  // Version * 10000 + Major * 100 + Minor
 
 // The plugin API's version number:
 
-#define APIVERSION  "1.7.16"
-#define APIVERSNUM   10716  // Version * 10000 + Major * 100 + Minor
+#define APIVERSION  "1.7.21"
+#define APIVERSNUM   10721  // Version * 10000 + Major * 100 + Minor
 
 // When loading plugins, VDR searches them by their APIVERSION, which
 // may be smaller than VDRVERSION in case there have been no changes to
@@ -36,7 +36,7 @@
 // plugins to work with newer versions of the core VDR as long as no
 // VDR header files have changed.
 
-#define LIEMIKUUTIO  130
+#define LIEMIKUUTIO  131
 
 // The MainMenuHook Patch's version number:
 #define MAINMENUHOOKSVERSION "1.0.1"
@@ -302,12 +302,12 @@ public:
   int CurrentChannel;
   int CurrentVolume;
   int CurrentDolby;
-  int InitialChannel;
   int InitialVolume;
   int ChannelsWrap;
   int EmergencyExit;
   int ShowRecDate, ShowRecTime, ShowRecLength, ShowProgressBar, MenuCmdPosition;
   int __EndData__;
+  cString InitialChannel;
   cSetup(void);
   cSetup& operator= (const cSetup &s);
   bool Load(const char *FileName);
