@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: cutter.c 2.5 2010/08/29 13:35:18 kls Exp $
+ * $Id: cutter.c 2.8 2011/08/21 11:08:08 kls Exp $
  */
 
 #include "cutter.h"
@@ -69,12 +69,14 @@ void cCuttingThread::Action(void)
 {
   cMark *Mark = fromMarks.First();
   if (Mark) {
+     SetPriority(19);
+     SetIOPriority(7);
      fromFile = fromFileName->Open();
      toFile = toFileName->Open();
      if (!fromFile || !toFile)
         return;
      fromFile->SetReadAhead(MEGABYTE(20));
-     int Index = Mark->position;
+     int Index = Mark->Position();
      Mark = fromMarks.Next(Mark);
      off_t FileSize = 0;
      int CurrentFileNumber = 0;
@@ -236,14 +238,14 @@ void cCuttingThread::Action(void)
 
            // Check editing marks:
 
-           if (Mark && Index >= Mark->position) {
+           if (Mark && Index >= Mark->Position()) {
               Mark = fromMarks.Next(Mark);
               toMarks.Add(LastIFrame);
               if (Mark)
                  toMarks.Add(toIndex->Last() + 1);
               toMarks.Save();
               if (Mark) {
-                 Index = Mark->position;
+                 Index = Mark->Position();
                  Mark = fromMarks.Next(Mark);
                  CurrentFileNumber = 0; // triggers SetOffset before reading next frame
                  cutIn = true;
@@ -281,11 +283,13 @@ bool cCutter::Start(const char *FileName, bool Overwrite)
      error = false;
      ended = false;
      cRecording Recording(FileName);
+
      cMarks FromMarks;
      FromMarks.Load(FileName);
-     cMark *First=FromMarks.First();
-     if (First) Recording.SetStartTime(Recording.start+(int(First->position/Recording.FramesPerSecond()+30)/60)*60);
-        cString evn = Recording.PrefixFileName('%');
+     if (cMark *First = FromMarks.First())
+        Recording.SetStartTime(Recording.Start() + (int(First->Position() / Recording.FramesPerSecond() + 30) / 60) * 60);
+
+     cString evn = Recording.PrefixFileName('%');
      if (!Overwrite && *evn && (access(*evn, F_OK) == 0) && !Interface->Confirm(tr("File already exists - overwrite?"))) {
         do {
            evn = PrefixVideoFileName(*evn, '%');
