@@ -10,7 +10,7 @@
  * and interact with the Video Disk Recorder - or write a full featured
  * graphical interface that sits on top of an SVDRP connection.
  *
- * $Id: svdrp.c 2.8 2010/01/17 12:23:31 kls Exp $
+ * $Id: svdrp.c 2.10 2011/08/27 10:43:18 kls Exp $
  */
 
 #include "svdrp.h"
@@ -219,9 +219,11 @@ const char *HelpPages[] = {
   "    image format defaults to JPEG.",
   "HELP [ <topic> ]\n"
   "    The HELP command gives help info.",
-  "HITK [ <key> ]\n"
+  "HITK [ <key> ... ]\n"
   "    Hit the given remote control key. Without option a list of all\n"
-  "    valid key names is given.",
+  "    valid key names is given. If more than one key is given, they are\n"
+  "    entered into the remote control queue in the given sequence. There\n"
+  "    can be up to 31 keys.",
   "LSTC [ :groups | <number> | <name> ]\n"
   "    List channels. Without option, all channels are listed. Otherwise\n"
   "    only the given channel is listed. If a name is given, all channels\n"
@@ -305,7 +307,10 @@ const char *HelpPages[] = {
   "    Turns the remote control on or off. Without a parameter, the current\n"
   "    status of the remote control is reported.",
   "RENR <number> <new name>\n"
-  "    Rename recording. Number must be the Number as returned by LSTR command.",
+  "    Rename the recording with the given number. Before a recording can be\n"
+  "    renamed, an LSTR command must have been executed in order to retrieve\n"
+  "    the recording numbers. The numbers don't change during subsequent RENR\n"
+  "    commands.\n"
   "SCAN\n"
   "    Forces an EPG scan. If this is a single DVB device system, the scan\n"
   "    will be done on the primary device unless it is currently recording.",
@@ -904,13 +909,28 @@ void cSVDRP::CmdHELP(const char *Option)
 void cSVDRP::CmdHITK(const char *Option)
 {
   if (*Option) {
-     eKeys k = cKey::FromString(Option);
-     if (k != kNone) {
-        cRemote::Put(k);
-        Reply(250, "Key \"%s\" accepted", Option);
-        }
-     else
-        Reply(504, "Unknown key: \"%s\"", Option);
+     char buf[strlen(Option) + 1];
+     strcpy(buf, Option);
+     const char *delim = " \t";
+     char *strtok_next;
+     char *p = strtok_r(buf, delim, &strtok_next);
+     int NumKeys = 0;
+     while (p) {
+           eKeys k = cKey::FromString(p);
+           if (k != kNone) {
+              if (!cRemote::Put(k)) {
+                 Reply(451, "Too many keys in \"%s\" (only %d accepted)", Option, NumKeys);
+                 return;
+                 }
+              }
+           else {
+              Reply(504, "Unknown key: \"%s\"", p);
+              return;
+              }
+           NumKeys++;
+           p = strtok_r(NULL, delim, &strtok_next);
+           }
+     Reply(250, "Key%s \"%s\" accepted", NumKeys > 1 ? "s" : "", Option);
      }
   else {
      Reply(-214, "Valid <key> names for the HITK command:");
@@ -1497,32 +1517,27 @@ void cSVDRP::CmdSCAN(const char *Option)
 
 void cSVDRP::CmdRENR(const char *Option)
 {
-  bool recordings = Recordings.Update(true);
-  if (recordings) {
-     if (*Option) {
-        char *tail;
-        int n = strtol(Option, &tail, 10);
-        cRecording *recording = Recordings.Get(n - 1);
-        if (recording && tail && tail != Option) {
-           char *oldName = strdup(recording->Name());
-           tail = skipspace(tail);
-           if (recording->Rename(tail)) {
-              Reply(250, "Renamed \"%s\" to \"%s\"", oldName, recording->Name());
-              Recordings.ChangeState();
-              Recordings.TouchUpdate();
-              }
-           else
-              Reply(501, "Renaming \"%s\" to \"%s\" failed", oldName, tail);
-           free(oldName);
+  if (*Option) {
+     char *tail;
+     int n = strtol(Option, &tail, 10);
+     cRecording *recording = Recordings.Get(n - 1);
+     if (recording && tail && tail != Option) {
+        char *oldName = strdup(recording->Name());
+        tail = skipspace(tail);
+        if (recording->Rename(tail)) {
+           Reply(250, "Recording \"%s\" renamed to \"%s\"", oldName, recording->Name());
+           ::Recordings.ChangeState();
+           ::Recordings.TouchUpdate();
            }
         else
-          Reply(501, "Recording not found or wrong syntax");
+           Reply(554, "Error while renaming recording \"%s\" to \"%s\"!", oldName, tail);
+        free(oldName);
         }
      else
-        Reply(501, "Missing Input settings");
+        Reply(550, "Recording \"%d\" not found%s", n, Recordings.Count() ? "" : " (use LSTR before renaming)");
      }
   else
-     Reply(550, "No recordings available");
+     Reply(501, "Invalid Option \"%s\"", Option);
 }
 						
 void cSVDRP::CmdSTAT(const char *Option)
@@ -1697,8 +1712,16 @@ bool cSVDRP::Process(void)
                  }
               else {
                  if (numChars >= length - 1) {
-                    length += BUFSIZ;
-                    cmdLine = (char *)realloc(cmdLine, length);
+                    int NewLength = length + BUFSIZ;
+                    if (char *NewBuffer = (char *)realloc(cmdLine, NewLength)) {
+                       length = NewLength;
+                       cmdLine = NewBuffer;
+                       }
+                    else {
+                       esyslog("ERROR: out of memory");
+                       Close();
+                       break;
+                       }
                     }
                  cmdLine[numChars++] = c;
                  cmdLine[numChars] = 0;
